'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _CMD  --> _CMD.CMD       |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012,2013 "Cluso99" (Ray Rodrick)                     |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' This OS module uses an included module "__MODULE.SPIN" to perform most of
''  the housekeeping - see this for history, details and acknowledgements.
''
'' RR20120317   This module does......
''                Issues the prompt and waits for a command to be entered, then
''                  loads/runs the command or binary file, or
''                  runs an internal command, or displays "use help" & the prompt.
''
'' RR20130802   Fix running ".BIN" files (clear OS upper hub ram & stop cogs) - didnt work!!!
''              Replace internal commands reboot, del, era, mv
''              Remove internal commands mount, unmount
'' RR20131022   Remove internal command togglelf (now external command LF)
'' RR20160104   Don't use internal " xxx.bin", "clear", "echo"


'#include "__MODULE.spin"        ' include the common code for OS modules 
#include "__MODULE.spin"

DAT
_ModuleStr    byte      "=== CMD ===",0        'module's name (string)

PRI executeCommand(stringPointer)  

    result := shellLine(string("SD:>"))                 ' wait for a command
    
    str.copy(result,@commandline)      ' save for later on if this is a command line with parameters eg "xmodem r filename.txt"
'================
    str.copy(@commandline,_hub#_pBuffer_A)              ' pass commandline to next program
'=================

    str.tokenizeString(@commandline)
    if strsize(@commandline) <> 0
      \runFilename(string("_"), @commandline, string(".CMD")) ' load/run file if found
'!    \runFilename(string(" "), @commandline, string(".BIN")) ' load/run file if found
      shellCommands(@commandline)                             ' try inbuilt commands
      PrintStringCR(string("Command not found, try help"))
                                                              'else just re-load _cmd.cmd
    
PRI runFilename(stringptr1, stringptr2, stringptr3) | r, errorString
' concatenate prefix+command+suffix to form a filename
' if file exists, load and run, else return
' uses @lineoftext2

    str.stringCopy(  @lineoftext2, stringptr1)
    str.stringAppend(@lineoftext2, stringptr2)
    str.stringAppend(@lineoftext2, stringptr3)
    str.trimString(  @lineoftext2)
'   printStringCR(   @lineoftext2)
    r := fat.openFile(@lineoftext2, "R")                ' if not found, will return to caller via abort
'   printStringCR(string("=file found="))

    'if ".BIN" file and it exists, set to clear OS from hub & stop cogs
    if (str.stringCompareCI(stringptr3, string(".BIN"))) == 0
'     printStringCR(string("=clear hub="))
      long[_hub#_HubFree] := _hub#_HUB_RAMSIZE          ' set top of hub free (all hub to be loaded)
      byte[_hub#_OS_Cogs] := 0                          ' set resident cogs (all to be stopped)
      if taf.partitionMounted
        taf.unmountPartition
        printStringCR(string("=unmount taf="))
'     if fat.partitionMounted
'       fat.unmountPartition
'       printStringCR(string("=unmount fat="))

    errorString := \fat.bootPartition(@lineoftext2)     ' if found, load and run module, else returns via abort
'   checkError(@lineoftext2, fat.partitionError, errorString) ' if error, display & abort
    if errorString <> 0
      printStringCR(string("error fat.bootPartition "))
      printStringCR(errorString)


    printStringCR(string("=why here="))
    waitcnt(clkfreq/10 + cnt)
    reboot

PRI shellCommands(stringPointer)

' programClear(stringPointer)
' programEcho(stringPointer)

  programChangeDirectory(stringPointer)
  programMakeDirectory(stringPointer)

  programChangeAttributes(stringPointer)
  programMakeFile(stringPointer)


PRI shellLine(prompt)

  PrintString(prompt)
  repeat
    result := GetKey            ' scan all inputs (keyboard/comport)
    PrintChar(result)           ' send to vga and to com port
    str.buildString(result)     ' build an input string
  while((result <> _carriageReturnCharacter) and (result <> _lineFeedCharacter) and (result <> _nullCharacter))

  return str.trimString(str.builtstring(true))

{  
PRI programClear(stringPointer)
  ifnot(str.stringCompareCI(string("clear"), stringPointer))
    waitcnt(clkfreq/1000 * 500 + cnt)                     ' delay 500ms
    printstring(string(_clearScreenCharacter, _homeCursorCharacter))
    return 0

PRI programEcho(stringPointer)
  ifnot(str.stringCompareCI(string("echo"), stringPointer))
    printstring(str.trimString(stringPointer += 5))
    crlf
    return 0
}

PRI programChangeDirectory(stringPointer)
  ifnot(str.stringCompareCI(string("cd"), stringPointer))
    stringPointer := str.tokenizeString(0)
    fat.changeDirectory(stringPointer)
    taf.changeDirectory(stringPointer)
    return 0

PRI programMakeDirectory(stringPointer)
  ifnot(str.stringCompareCI(string("mkdir"), stringPointer))
    fat.newDirectory(str.tokenizeString(0))
    return 0

PRI programChangeAttributes(stringPointer)
  ifnot(str.stringCompareCI(string("chmod"), stringPointer))
    fat.changeAttributes(str.tokenizeString(0), str.tokenizeString(0))
    return 0

PRI programMakeFile(stringPointer)
  ifnot(str.stringCompareCI(string("mkfile"), stringPointer))
    fat.newFile(str.tokenizeString(0))
    return 0


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}