'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _RUN  --> _RUN.CMD       |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2012,2013 "Cluso99" (Ray Rodrick)                     |
'' |  Modifications:                                                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' This OS module uses an included module "__MODULE.SPIN" to perform most of
''  the housekeeping - see this for history, details and acknowledgements.
''
'' This module......
''      Runs a "xxx.BIN" program. (because _cmd.cmd does not work!!)
''
'' RR20130802   Fix running ".BIN" files
'' RR20160104   stringConcatenate -> stringAppend


'#include "__MODRUN.spin"        ' include the common code for OS modules 
#include "__MODRUN.spin"

VAR
  byte  filename0[13]                           ' FAT source filename       (string)

DAT
_ModuleStr    byte      "=== RUN ===",0        'module's name (string)

PRI executeCommand(stringPointer) | sector[512], errorString, bytes, n

  'get the parameters
  str.stringCopy(@filename0, str.tokenizeString(0))     ' source filename

  if strsize(@filename0) == 0                           ' if no parameters, display usage
    printStringCR(string(" Run an 'xxx.BIN' file..."))
    printStringCR(string("  RUN <filename>"))
    return 0

  \runFilename(string(" "), @filename0, string(".BIN")) ' load/run file if found
  
PRI runFilename(stringptr1, stringptr2, stringptr3) | r, errorString
' concatenate prefix+command+suffix to form a filename
' if file exists, load and run, else return
' uses @lineoftext2
'    str.copy(stringptr1,@lineoftext2)
    str.stringCopy(  @lineoftext2, stringptr1)
    str.stringAppend(@lineoftext2, stringptr2)
    str.stringAppend(@lineoftext2, stringptr3)
    str.trimString(  @lineoftext2)
'    str.copy(@lineoftext2, @filename0)
    str.stringCopy(  @filename0, @lineoftext2)

'-----------------------------------------------------------------
' Open the FAT source file
    errorString := \fat.openfile(@filename0, "R")       ' open source file
    checkError(@filename0, fat.partitionError, errorString) ' if error, display & abort

    printString(string("i) Running File "))
    printStringCR(@filename0)


    long[_hub#_HubFree] := _hub#_HUB_RAMSIZE            ' set top of hub free (all hub to be loaded)
    byte[_hub#_OS_Cogs] := 0                            ' set resident cogs (all to be stopped)
{
      if taf.partitionMounted
        taf.unmountPartition
        printStringCR(string("=unmount taf="))
}

    errorString := \fat.bootPartition(@filename0)       ' if found, load and run module, else returns via abort
'   checkError(@filename0, fat.partitionError, errorString) ' if error, display & abort
    if errorString <> 0
      printStringCR(string("error fat.bootPartition "))
      printStringCR(errorString)

    printStringCR(string("=why here?="))
    waitcnt(clkfreq/10 + cnt)
    reboot


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}