'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _SPHINX  --> _SPHINX.CMD |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2012-2016 "Cluso99" (Ray Rodrick)                     |
'' |  Modifications: see below for original authors and acknowledgements      |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' This OS module uses an included module "__MODULE.SPIN" to perform most of
''  the housekeeping - see this for history, details and acknowledgements.
''
'' This module......
''      Runs programs _LEX, _CODEGEN & LINK (spin/pasm compiler for propeller)
''        These programs were formerly part of Sphinx written by Michael Park.
''
'' +--------------------------------------------------------------------------+
'' | SphinxOS   An Operating System for the Propeller Chip             v0.xxx |
'' +--------------------------------------------------------------------------+
'' |  Authors:                                                                |
'' |  (c) 2009,2010      Michael Park (mpark)             (original Sphinx)   |
'' |                     Radical Eye Software (rokicki?)  (fsrw objects)      |
'' |                     Lonesock & Rokicki               (fsrw objects)      |
'' |                     Andy Schenk (Ariba)              (sxkb_pc, sxtv_pc)  |
'' |                     Ray Rodrick (Cluso99)            (TriBlade/RamBlade) |
'' |                     Ray Rodrick (Cluso99)            (addn's to SphinxOS)|
'' |  (c) 2015,2016      Ray Rodrick (Cluso99)            (include in PropOS) |
'' |  License            MIT License - See end of file for terms of use       |
'' +--------------------------------------------------------------------------+
''
''  Simple command line
''
''2009
''April
''  15    Adding file transfer to/from PC via serial
''  16    0.02
''        SendFile bugfix (while blksize == 1024)
''        Tries to run .bin before .eep now.
''June
''  4     Changing name from cl to sphinx. Converting to use sxfs, sxtv.
''  15    sxkb; adding c and cl commands
''  23    Corrected clock mode calculation.
''
''Juli 2009
'' changed for (Prop-)Terminal instead TV and PS2 by Andy Schenk
'' added GET <file> and PUT <file> for File up/download (not working yet)
''
'' RR20100117 Cluso99: allow specific pins for SD card (passed in basepin)
'' RR20100119 Cluso99: rename SphinxOS_xxx.spin; add isx_in.spin and isx.out.spin for standard I/O
'' RR20100203          v007 failed so retry taking 006 --> 006a
'' RR20100203          pass rendezvous at start
'' RR20100217          v006e
''                     v006n AuxIn/AuxOut 1-pin Kbd & TV working
'' RR20100222          v006q add OPTION to select the I/O drivers for StdIn/StdOut & AuxIn/AuxOut
'' RR20100222          v006s modify "sxfs2.spin" to only load 56 sectors to hub ram (was 63)
''                            this prevents overwriting the screen buffer & rendezvous area
''                     v006t add OPTIONs 2 & 3
''                     v006u RamBlade - kbd does not work at 6.5MHz, ok at 6MHz
'' RR20100224          v011  RamBlade (kbd not wkg @ 6.5MHz - to fix later), back to 6MHz
'' RR20100226          v012  RamBlade - only start StdI/O Driver
'' RR20100226          v015  RamBlade (use OPTION = 0 compiled with 6.5MHz)
''                                    (use OPTION = 1 at 6MHz only for now as Keyboard does not work at 6.5MHz)
'' RR20151228          v020  P8XBlade2 (use OPTION = 0 compiled with 12MHz PLL8)
'' RR20151231          v021  mod to include as PropOS _SPHINX.cmd



'WARNING: Somewhere there is a dependancy preventing relocation of upper hub definitions.


 '#include "__MODULE.spin"        ' include the common code for OS modules 
#include "__MODULE.spin"

VAR



DAT
_ModuleStr    byte      "== SPHINX==",0        'module's name (string)


PRI executeCommand(StringPointer)

  'get the parameters
  str.stringCopy(@filename1, str.tokenizeString(0))     ' fat filename (optional)
' str.stringCopy(@param, str.tokenizeString(0))         ' optional -s parameter

  if (filename1[0] == "?" and filename1[1] == 0)        ' help parameter...
    printStringCR(string(" Display the file directory"))
    printStringCR(string("  DIR [mask]"))
    return 0

  if strsize(@filename1) > 0                            ' if exists, prepare fname 8+3 for compare
    ExpandWildcards("?")                                ' expands filename to uppercase(fname) as 8+3 string
    str.stringCopy(@fname2, @fname1)                    ' save then compare-to fname
  str.stringCopy(@filename2, @filename1)

PUB Main | err, p, ch, firsttime, i


  waitcnt(clkfreq*1 + cnt)
'--------------------------------------------------------------------------------------------------
  'start standard I/O interface...
  term.start(hub#_StdOut)                               ' StdOut driver interface
  kb.start(hub#_StdIn)                                  ' StdIn  driver interface
'--------------------------------------------------------------------------------------------------
  if OPTION == 0                                        ' OPTION = 0 FOR PC AS STANDARD I/O
    'start standard input/output driver... (checks if running)
    firsttime := sio.start(hub#_StdIn,hub#_StdOut,hw#serPins,hw#baud)   ' StdIn & StdOut cog driver (PC-Serial I/O Driver)
'--------------------------------------------------------------------------------------------------
  if OPTION == 1                                        ' OPTION = 1 FOR 1-PIN KBD & TV AS STANDARD I/O
    'check if Standard Output driver running
    long[hub#_StdOut] := $47 | %100                     ' send BELL  "G"
    waitcnt(clkfreq/10 + cnt)                           ' wait 100ms
    if long[hub#_StdOut] <> 0                           ' char not taken?
      firsttime := true
      'start standard output driver...
      long[hub#_StdOut] := hub#_Screen << 8 | hw#tvPinc       ' pass hub screen address & tv pin#
      optv.start(hub#_StdOut)                                 ' StdOut cog driver (1-pin TV Driver)
      'start standard input  driver...
      long[hub#_StdIn]  := hw#ktime << 16 | hw#kdPin          ' pass kbd time and pin#
      opkb.start(hub#_StdIn)                                  ' StdIN  cog driver (1-pin Keyboard Driver)
      waitcnt(clkfreq/10 + cnt)
      repeat until long[hub#_StdIn] == 0                      ' wait until cleared
    else                                                ' taken, so cog running
      firsttime := false
'--------------------------------------------------------------------------------------------------
  if firsttime                                          ' firsttime: we started the drivers in the cog(s)
    term.out(0)
    term.out(13)                                        ' because 0 doesn't clear screen in some drivers
    term.str(@ProgID)
    term.str(string("SD="))
    term.dec(hw#spiDO)
    term.out(",")
    term.dec(hw#spiCLK)
    term.out(",")
    term.dec(hw#spiDI)
    term.out(",")
    term.dec(hw#spiCS)
    if OPTION == 0                                      ' OPTION = 0 FOR PC AS STANDARD I/O
      term.str(string(" Serial="))
      term.dec(hw#rxPin)
      term.out(",")
      term.dec(hw#txPin)
    if OPTION == 1                                      ' OPTION = 1 FOR KBD/TV AS STANDARD I/O
      term.str(string(" Kbd/TV="))
      term.dec(hw#kdPin)
      term.out(",")
      term.dec(hw#TVPinc)
    term.out(" ")
    term.dec(clkfreq/1000000)
    term.str(string("MHz"))
    term.out(13)
'--------------------------------------------------------
  if firsttime
    'start SD driver(s)                                 ' START SD DRIVERS
    sxfs.start(hw#sdPins)                               ' SD cog Driver(s)
'--------------------------------------------------------
  if not firsttime
    'msg to show we are here
    term.out(13)
    term.str(@ProgID)
'--------------------------------------------------------
' ALL INITIALISATION COMPLETE
'--------------------------------------------------------

  repeat
    f.Close
    err := \DoIt
    if err
      if err > 0
        term.str( err )
      else
        term.str( string("Error ") )
        term.dec( err )
      term.out( 13 )
    else
      term.str( string("Normal exit") )                 ' This never happens


pri DoIt | ch
  term.str(string( "SD>_",8 ))
  repeat
    if kb.peek
      EditCmdLine
      Execute
      term.str(string( "??>_",8 ))                 '<==== never executed

con
  MAXCMDLINELENGTH = 78 ' some arbitray limit.
  CR     = $0d
  BKSP   = $c8

var
  byte cmdLine[MAXCMDLINELENGTH+1]                      ' +1 for null terminator
  long cmdLineLength

pri EditCmdLine | key
  cmdLineLength~                ' cursor is always at end of command line

  repeat
    case key := kb.in
      $20..$7f:
        if cmdLineLength < MAXCMDLINELENGTH
          cmdLine[ cmdLineLength++ ] := key
          term.out( key )
          term.out( "_" )        
          term.out( 8 )        
      CR:
        term.out( " " )
        term.out( 13 )
        cmdLine[ cmdLineLength ]~
        return true
      BKSP:
        if cmdLineLength
          --cmdLineLength
          term.out( " " )
          term.out( 8 )
          term.out( 8 )
          term.out( "_" )
          term.out( 8 )

con
TIMEOUT = 500

var
  byte buffer[1024]
       
pri Execute | n, p, q, run, c, b, len, i
  c~
  ifnot n := ParseCmdLine
    return
  p := @cmdLine
  ToUpper(p)
  
  if run := strcomp( @cmdLine, string("RUN") ) 
    --n
    p += strsize(p) + 1
  elseif strcomp( @cmdLine, string("C") )
    c := 1
    --n                                                                                                               
    p += strsize(p) + 1                                                                     ' command arg1 arg2...    
  elseif strcomp( @cmdLine, string("CL") )                                                  ' run command arg1 arg2...
    c := 2                                                                                  ' c arg1 arg2...          
    --n                                                                                     ' cl arg1 arg2...         
    p += strsize(p) + 1

  'added by AS
  elseif strcomp( @cmdLine, string("EXIT") )
    reboot
  elseif strcomp( @cmdLine, string("GET") )
    p += strsize(p) + 1
    n := strsize(p)
    if n
      q := p
      if byte[q] == "*"
        q += strsize(q) + 1
'      if f.open( q,"W") <> 0  or strsize(p)==0
'        abort string(" - can't open")
      term.out(0)
      term.str(p)                           'send filename
      term.str(string(5,$82))               'initiate download
      term.out(n)
      term.out("f")
      repeat 1000
        if b := kb.peek
          quit
        waitcnt(clkfreq>>9+cnt)
      ifnot b
        abort string(" - File not found")
      len := kb.in                         'PropTerm sends first the FileLength (4 bytes)
      b := kb.in
      len |= b<<8
      b := kb.in
      len |= b<<16
      b := kb.in
      len |= b<<24
      n :=  0                              'counter for received bytes
      repeat while n < len
        term.out(".")                      'trigger next 16 bytes
        repeat i from 0 to 15              'receive 16 bytes
          if n+i => len                    'quit if length reached
            quit
          b := kb.in
          buffer[i] := b                   'store in buffer
'        f.write(@buffer,16)                'write the buffer to the SD card
        n += 16
'      f.close
      term.out(13)
      term.dec( n )
      abort string(" bytes received", 13)
  elseif strcomp( @cmdLine, string("PUT") )
    p += strsize(p) + 1
    abort string(" not yet implemented", 13)
    
  ifnot n                                                               
    abort string("No file specified")                                   
                                                                        
  f.Open( string("args.d8a"), "W" )                     ' write args    
  if c == 0
    q := p + strsize(p) + 1                             ' for run and plain command, skip first arg (command name)
    --n
    f.Write( @n, 1 )
    repeat n
      f.Write( q, strsize(q) + 1 )
      q += strsize(q) + 1
  else                                                  ' for c and cl,
    ++n                                                 '  add extra arg (/c or /l)
    f.Write( @n, 1 )
    repeat n-1
      f.Write( p, strsize(p) + 1 )
      p += strsize(p) + 1
    if c == 1
      f.Write( string("/c"), 3 )
    else
      f.Write( string("/l"), 3 )
        
  n~
  f.Write( @n, 1 )              ' final terminating null    
  f.Close

  if c 
    if f.Open( string("lex.bin"), "R" ) <> 0
      abort string("lex.bin not found")
    f.Execute( 0 )  
    

  if EndsWith( p, string(".BIN") ) or EndsWith( p, string(".EEP") )
    if f.Open( p, "R" ) <> 0
      term.str( p )
      abort string(" -- not found")
  else
    if strsize(p) > MAXFILENAMELENGTH - 4
      term.str( p )
      abort string(" -- filename too long")
    bytemove( @filename, p, strsize(p) + 1 )
    bytemove( @filename + strsize(p), string(".BIN"), 5 )
    if f.Open( @filename, "R" ) <> 0
      bytemove( @filename + strsize(p), string(".EEP"), 5 )
      if f.Open( @filename, "R" ) <> 0
        term.str( p )
        abort string(" -- no .bin or .eep found")
  f.Execute( run )

pri ToUpper( p )
  repeat while byte[p]
    if "a" =< byte[p] and byte[p] =< "z"
      byte[p] += "A" - "a"
    ++p

pri EndsWith( p, s )
  if strsize(p) < strsize(s)
    return false
  p += strsize(p) - strsize(s)
  repeat while byte[p]
    if byte[p++] <> byte[s++]
      return false
  return true

con
  MAXFILENAMELENGTH = 12 ' 8 + . + 3
  FILENAMEBUFFERSIZE = MAXFILENAMELENGTH + 1 ' 8 + . + 3 + null
  CMDNAMEBUFFERSIZE = 9 ' 8 + null
  
var
  byte filename[MAXFILENAMELENGTH+1] ' 8 + . + 3 + null
  
pri ParseCmdLine : n | pSrc, pDst, state
{{
  Converts cmdLine from " ab  de   fg  \" to "ab\de\fg\" ('\' = null).
  Returns # of "words".
}}
{
  state     whitespace                null                      other char
  0         ++pSrc;                   append \0; quit           append char; => 1
  1         append \0; ++n; => 0      append \0; ++n; quit      append char;
}
  pSrc := @cmdLine
  pDst := @cmdLine
  state~
  repeat
    if state == 0
      if byte[pSrc] == " "
        ++pSrc
      elseif byte[pSrc] == 0
        byte[pDst++]~
        quit
      else
        byte[pDst++] := byte[pSrc++]
        state := 1
    elseif state == 1
      if byte[pSrc] == " "
        ++pSrc
        byte[pDst++]~
        ++n
        state~
      elseif byte[pSrc] == 0
        byte[pDst++]~
        ++n
        quit
      else
        byte[pDst++] := byte[pSrc++]

{{
Copyright (c) 2009 Michael Park
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    |
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
        